/*
	These are simple defaults for your project.
 */

world
	fps = 25		// 25 frames per second
	icon_size = 32	// 32x32 icon size by default

	view = 6		// show up to 6 tiles outward from center (13x13 view)


// Make objects move 8 pixels per tick when walking

var
{
	list/g_listOfFeedbackCategories = list("Bug", "Feature Request", "Other");
	list/g_listOfFeedbackSeverities = list("High", "Medium", "Low", "Whatever");
	list/g_listOfFeedbackStatuses = list("New", "Resolved");
}

mob
{
	var
	{
		FeedbackController/_feedbackController
		// TODO: Init on form open
		list/_listOfTempDropDownItems = new();
		list/_listOfOpenDropDowns = new();
		Feedback/_loadedAdminFeedback
		_selectedStatus = "All"
		_showFeatureFeedback = 1
		_showBugFeedback = 1
		_showOtherFeedback = 1
		_showSeverityHighFeedback = 1
		_showSeverityMediumFeedback = 1
		_showSeverityLowFeedback = 1

	}

	Login()
	{
		..()
	}

	verb
	{
		AdminFeedbackWindowClosed()
		{
			HandleAdminFeedbackWindowClosing()
		}

		DropDownListClicked(var/t as text)
		{
			HandleDropDownClickEvent(t)
		}

		ItemSelected(var/t as text)
		{
			HandleDropDownListSelection(t)
		}

		FeedbackSave()
		{
			HandleSavingUsersFeedback()
		}

		FeedbackEdit()
		{
			SaveEditedFeedback()
		}

		RefreshFeedback()
		{
			PopulateFeedbackOutput()
		}

		FeedbackCancel()
		{
			CancelFeedbackDialog()
		}

		UpdateFilters(var/statusFilter = null as text)
		{
			HandleFilterUpdates(statusFilter)
		}

		ShowFeedbackWindow()
		{
			DisplayFeedbackDialog()
		}

		ShowAdminFeedbackWindow()
		{
			DisplayAdminFeedbackDialog()
		}
	}

	proc
	{
		HandleAdminFeedbackWindowClosing()
		{
			_loadedAdminFeedback = null
			// _selectedStatus // Don't reset so user has the same filter for this session (typically new).
			_listOfOpenDropDowns.Cut()
			_listOfTempDropDownItems.Cut()
			_feedbackController._listOfFeedback.Cut()
			// Don't destroy the controller because it's used by the FeedbackWindow and the AdminFeedbackWindow
		}

		HandleDropDownListSelection(var/t)
		{
			var/list/listOfDelimitedValues = str_Split(t, ".")
			var/windowName = listOfDelimitedValues[1]
			var/controlName = listOfDelimitedValues[2]
			var/selectedValue = listOfDelimitedValues[3]
			winset(src, "[windowName].[controlName]DdlLabel", "text=\"[selectedValue]\"");
			world << "[src] selected [selectedValue]";
			HideDropDowns();
		}

		CancelFeedbackDialog()
		{
			winset(src, "FeedbackWindow", "is-visible=false");
		}

		HandleFilterUpdates(var/statusFilter)
		{
			var/windowName = "AdminFeedbackWindow"
			ResetForm(windowName)
			if (statusFilter)
			{
				_selectedStatus = statusFilter
			}

			_showFeatureFeedback = winget(src, "[windowName].ShowFeatureCheckbox", "is-checked") == "true"
			_showBugFeedback = winget(src, "[windowName].ShowBugsCheckbox", "is-checked") == "true"
			_showOtherFeedback = winget(src, "[windowName].ShowOtherCheckbox", "is-checked") == "true"
			_showSeverityHighFeedback = winget(src, "[windowName].ShowSeverityHighCheckbox", "is-checked") == "true"
			_showSeverityMediumFeedback = winget(src, "[windowName].ShowSeverityMediumCheckbox", "is-checked") == "true"
			_showSeverityLowFeedback = winget(src, "[windowName].ShowSeverityLowCheckbox", "is-checked") == "true"
			PopulateFeedbackOutput(1)
		}

		HandleSavingUsersFeedback()
		{
			CreateNewFeedback()
			alert("Feedback Submitted!", "OK")
			ResetForm("FeedbackWindow")
		}

		ResetForm(var/windowName)
		{
			winset(src, "[windowName].FeedbackTitleInput", "text=\"\"");
			winset(src, "[windowName].FeedbackCategoryDdlLabel", "text=\"Bug\"");
			winset(src, "[windowName].FeedbackSeverityDdlLabel", "text=\"Medium\"");
			winset(src, "[windowName].FeedbackReportInput", "text=\"\"");

			if (windowName == "AdminFeedbackWindow")
			{
				winset(src, "[windowName].FeedbackCreatedByInput", "text=\"\"");
				winset(src, "[windowName].FeedbackStatusDdlLabel", "text=\"New\"");
			}
		}

		DisplayFeedbackDialog()
		{
			var/windowName = "FeedbackWindow"
			// Why not save the users data in case they need some back and forth.
			// ResetForm(windowName)

			if (!_feedbackController)
			{
				InitFeedbackController()
			}

			winset(src, windowName, "is-visible=true");
		}

		DisplayAdminFeedbackDialog()
		{
			InitAdminFeedbackWindow()
			winset(src, "AdminFeedbackWindow", "is-visible=true");
		}

		InitAdminFeedbackWindow()
		{
			if (!_feedbackController)
			{
				InitFeedbackController()
			}

			LoadFeedbackCache()
			OpenAdminFeedbackWindow()
			PopulateFeedbackOutput(1)
		}

		LoadFeedback(var/feedbackIdAsText, var/windowName="AdminFeedbackWindow")
		{
			var/Feedback/feedback = LoadFeedbackById(text2num(feedbackIdAsText))

			if (!feedback)
			{
				world.log << "Critical Failure! Loading Feedback Id: [feedbackIdAsText]"
			}

			_loadedAdminFeedback = feedback
			winset(src, "[windowName].FeedbackTitleInput", "text=\"[feedback._title]\"");
			winset(src, "[windowName].FeedbackCategoryDdlLabel", "text=\"[feedback._category]\"");
			winset(src, "[windowName].FeedbackSeverityDdlLabel", "text=\"[feedback._severity]\"");
			winset(src, "[windowName].FeedbackReportInput", "text=\"[feedback._report]\"");
			winset(src, "[windowName].FeedbackCreatedByInput", "text=\"[feedback._createdBy]\"");
			winset(src, "[windowName].FeedbackStatusDdlLabel", "text=\"[feedback._status]\"");
			PopulateFeedbackOutput()
		}

		LoadFeedbackById(var/id)
		{
			for(var/Feedback/feedback in _feedbackController._listOfFeedback)
			{
				if (feedback._id == id)
				{
					return feedback
				}
			}

			// Should be a critical failure since we always expect to find what we're looking for.
			return null
		}

		CreateNewFeedback()
		{
			var/windowName = "FeedbackWindow"
			var/Feedback/feedback = new()
			var/title = winget(src, "[windowName].FeedbackTitleInput", "text");
			var/category = winget(src, "[windowName].FeedbackCategoryDdlLabel", "text");
			var/severity = winget(src, "[windowName].FeedbackSeverityDdlLabel", "text");
			var/report = winget(src, "[windowName].FeedbackReportInput", "text");
			feedback._id = ++_feedbackController._feedbackCounter
			feedback._title = title;
			feedback._category = category;
			feedback._severity = severity;
			feedback._report = report;
			feedback._status = g_listOfFeedbackStatuses[1] // 'New' but in case it changes. First is default "new" status
			feedback._createdBy = ckey
			_feedbackController._listOfFeedback.Add(feedback)
			SaveFeedbackToFile(feedback)
		}

		SaveEditedFeedback()
		{
			if (!_loadedAdminFeedback)
			{
				// User tried to save with nothing loaded
				return
			}

			var/windowName = "AdminFeedbackWindow"
			var/title = winget(src, "[windowName].FeedbackTitleInput", "text");
			var/category = winget(src, "[windowName].FeedbackCategoryDdlLabel", "text");
			var/severity = winget(src, "[windowName].FeedbackSeverityDdlLabel", "text");
			var/report = winget(src, "[windowName].FeedbackReportInput", "text");
			var/status = winget(src, "[windowName].FeedbackStatusDdlLabel", "text");
			_loadedAdminFeedback._title = title;
			_loadedAdminFeedback._category = category;
			_loadedAdminFeedback._severity = severity;
			_loadedAdminFeedback._report = report;
			_loadedAdminFeedback._status = status;
			SaveFeedbackToFile(_loadedAdminFeedback)
			PopulateFeedbackOutput()
		}

		CancelFeedback(var/windowName)
		{
			winset(src, windowName, "is-visible=false");
		}

		HandleDropDownClickEvent(var/delimitedSelection)
		{
			var/list/listOfDelimitedValues = str_Split(delimitedSelection, ".")
			var/windowName = listOfDelimitedValues[1]
			var/controlName = listOfDelimitedValues[2]
			var/list/listOfValues = GetListOfValues(controlName);
			HideDropDowns();
			var/label = winget(src, "[windowName].[controlName]DdlLabel", "pos;size")
			var/plist[] = params2list(label)
			var/x = text2num(copytext(plist["pos"], 1, findtext(plist["pos"], ",")));
			var/y = text2num(copytext(plist["pos"], findtext(plist["pos"], ",") + 1))
			var/h = text2num(copytext(plist["size"], findtext(plist["size"], "x") + 1))
			var/w = text2num(copytext(plist["size"], 1, findtext(plist["size"], "x")))
			var/list/OutlineLabelParams = new;
			OutlineLabelParams["type"] = "Label";
			OutlineLabelParams["parent"] = windowName;
			OutlineLabelParams["size"] = "[w+3]x[(h * listOfValues.len)+2]";
			OutlineLabelParams["index"] = 1
			OutlineLabelParams["pos"] = "[x-1],[y+h-1]"
			OutlineLabelParams["border"] = "Line"

			//
			// Create each item in the list
			//
			for (var/i = 1; i <= listOfValues.len; i++)
			{
				var/t = listOfValues[i];
				_listOfOpenDropDowns += "[controlName]DdlButton_[t]";
				_listOfOpenDropDowns += "[controlName]DdlLabel_[t]";

				var/list/ButtonParams = new;
				ButtonParams["type"] = "Button";
				ButtonParams["command"] = "ItemSelected [windowName].[controlName].[listOfValues[i]]";
				ButtonParams["parent"] = windowName
				ButtonParams["size"] = plist["size"];
				ButtonParams["pos"] = "[x],[y + (h * i)]"

				winset(usr, "[windowName].[controlName]DdlButton_[t]", list2params(ButtonParams))

				var/list/LabelParams = new;
				LabelParams["type"] = "Label"
				LabelParams["text"] = "[t]";
				LabelParams["parent"] = windowName;
				LabelParams["size"] = plist["size"];
				LabelParams["align"] = "left";
				LabelParams["pos"] = "[x],[y + (h * i)]"
				LabelParams["is-disabled"] = "true"
				LabelParams["background-color"] = rgb(79,79,79)
				LabelParams["text-color"] = "white"

				winset(usr, "[windowName].[controlName]DdlLabel_[t]", list2params(LabelParams))
			}
		}

		GetListOfValues(var/controlName)
		{
			if (controlName == "FeedbackCategory")
			{
				return g_listOfFeedbackCategories;
			}
			else if (controlName == "FeedbackSeverity")
			{
				return g_listOfFeedbackSeverities;
			}
			else if (controlName == "FeedbackStatus")
			{
				return g_listOfFeedbackStatuses;
			}
			else
			{
				world.log << "GetListOfValues() - FAIL! controlName: '[controlName]'";
			}
		}

		HideDropDowns()
		{
			for (var/i = 1; i <= _listOfOpenDropDowns.len; i++)
			{
				winset(src, _listOfOpenDropDowns[i], "parent=");
			}

			_listOfOpenDropDowns.Cut();
		}

		PopulateFeedbackOutput(var/isLoadingFirstEntry = 0)
		{
			usr << output(null, "FeedbackOutput");
			var/hasLoadedFirstEntry = 0

			for (var/Feedback/feedback in _feedbackController._listOfFeedback)
			{
				var/color = "white"
				if (feedback == _loadedAdminFeedback)
				{
					color = "red"
				}

				if (_selectedStatus != "All" && _selectedStatus != feedback._status)
				{
					continue;
				}

				if (!_showFeatureFeedback && feedback._category == "Feature")
				{
					continue;
				}

				if (!_showBugFeedback && feedback._category == "Bug")
				{
					continue;
				}

				if (!_showOtherFeedback && feedback._category == "Other")
				{
					continue;
				}

				if (!_showSeverityHighFeedback && feedback._severity == "High")
				{
					continue;
				}

				if (!_showSeverityMediumFeedback && feedback._severity == "Medium")
				{
					continue;
				}

				if (!_showSeverityLowFeedback && feedback._severity == "Low")
				{
					continue;
				}

				usr << output("<a href=\"?[feedback._id]\" style=\"color:[color]\">\[[feedback._id]\] \[[feedback._category]\] \[[feedback._status]\] \[[feedback._severity]\] - [feedback._title]</a>", "FeedbackOutput")

				if(isLoadingFirstEntry && !hasLoadedFirstEntry)
				{
					hasLoadedFirstEntry = 1
					LoadFeedback("[feedback._id]")

					// LoadFeedback is going to call back into
					// PopulateFeedbackOutput and overwrite everything this is doing
					// so stop here.
					break;
				}
			}
		}

		InitFeedbackController()
		{
			LoadFeedbackController()

			if (!_feedbackController._listOfFeedback)
			{
				_feedbackController._listOfFeedback = new()
			}
		}

		LoadFeedbackCache()
		{
			// Rebuilding it.
			_feedbackController._listOfFeedback.Cut()

			for(var/i = 1; i <= _feedbackController._feedbackCounter; i++)
			{
				var/Feedback/feedback = LoadFeedbackFromFile(i)
				_feedbackController._listOfFeedback.Add(feedback)
			}
		}

		UnloadFeedbackCache()
		{
			_feedbackController._listOfFeedback.Cut()
		}

		OpenAdminFeedbackWindow()
		{
			winset(src, "AdminFeedbackWindow", "is-visible=true");
		}

		SaveFeedbackToFile(var/Feedback/feedback)
		{
			var/savefile/F = new("gamedata\\feedback\\feedback_[feedback._id]")
			feedback.Write(F)
			SaveFeedbackController()
		}

		LoadFeedbackFromFile(var/feedbackId)
		{
			var/savefile/F = new("gamedata\\feedback\\feedback_[feedbackId]")
			var/Feedback/feedback = new()
			feedback.Read(F)
			return feedback
		}

		SaveFeedbackController()
		{
			var/savefile/F = new("gamedata\\FeedbackController")
			_feedbackController.Write(F)
		}

		LoadFeedbackController()
		{
			var/savefile/F = new("gamedata\\FeedbackController")

			if (!_feedbackController)
			{
				_feedbackController = new()
			}

			_feedbackController.Read(F)

			if (!_feedbackController)
			{
				_feedbackController = new()
			}
		}
	}
}

obj
	step_size = 8
